/*
 * fancyenum.h
 *
 *  Created on: 17.05.2009
 *      Author: wglas
 */

#ifndef FANCYENUM_H_
#define FANCYENUM_H_

#include <fancyserializable.h>

namespace Fancy {

/**
 * A virtual base class for JAVA enum objects.
 *
 * Implementations of this class should provide the following members:
 * @verbatim
 *   enum enum_t {
 *     ENUM_VALUE0,
 *     ENUM_VALUE1,
 *     ....
 *   };
 *
 *   const char* getSerialID() const;
 *   Serializable::ptr_t resolveRead(int32_t v) const;
 *   int32_t getMaxValue() const;
 *   static ptr_t newInstance(enum_t e);
 *   static Serializable::ptr_t newSerializableInstance();
 *   static const char *getSerialID_s();
 * @endverbatim
 */
class FANCYBASE_API Enum: public Serializable {

    int32_t v;

protected:
    Enum(int32_t _v) : v(_v) {}

    ~Enum();

    /**
     * Resolve a static enum instance.
     */
    virtual Serializable::ptr_t resolveRead(int32_t v) const = 0;

public:
    typedef ObjPtr<Enum> ptr_t;

    /**
     * Return the C++ enum value.
     */
    int32_t getValue() const { return this->v; }

    /**
     * The C++ enum value with the highest ordinary integral value.
     */
    virtual int32_t getMaxValue() const = 0;

    std::string toString() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    // implementation of Fancy::Serializable
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    Serializable::ptr_t readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);
};
}

#endif /* FANCYENUM_H_ */
