// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancynumericconverter.h 909 2011-05-27 21:20:49Z fancylibs $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** Classes for string conversions.
**
*****************************************************************************/

#ifndef FANCYBASE_NUMERICCONVERTER_H
#define FANCYBASE_NUMERICCONVERTER_H

#include <string>
#include <locale>
#include <fancybase.h>
#include <fancyint.h>

namespace Fancy {

  /**
   * A converter for string representations of numbers.
   * @c NumericConverter instances are expected to be instantiated on the
   * stack as a local variable.
   *
   * Parse errors are reported by throwing a @c Fancy::Exception.
   */
  class FANCYBASE_API NumericConverter
  {
  public:
    /**
     * Instantiate a new instance for the default locale.
     * The default locale my be set using @c std::locale::global().
     *
     * If you need a numeric converter for the classical "C" locale, use
     * @verbatim NumericConverter converter(std::locale::classic())
     * @endverbatim
     */
    NumericConverter();

    /**
     * Instantiate a new instance for the given locale.
     */
    NumericConverter(const std::locale &l);

    /**
     * public destructor.
     */
    ~NumericConverter();

    /**
     * Change the locale used by the converter to the given one.
     */
    void imbue(const std::locale &l);

    /**
     * Parse a boolean value. Valid boolean strings are
     * @verbatim "0", "1", "false", "true", "yes", "no"
     * @endverbatim
     */
    bool     toBool(const std::string &s) const;

    /**
     * Parse a decimal string representation of an integral number.
     */
    int      toInt(const std::string &s) const;

    /**
     * Parse a decimal string representation of an unsigned integral number.
     */
    unsigned toUInt(const std::string &s) const;

    /**
     * Parse a decimal string representation of an integral number.
     */
    int64_t  toInt64(const std::string &s) const;

    /**
     * Parse a decimal string representation of an unsigned integral number.
     */
    uint64_t toUInt64(const std::string &s) const;

    /**
     * Parse a decimal string representation of an double precision floating point number.
     */
    double   toDouble(const std::string &s) const;

    /**
     * Parse a decimal string representation of a single precision floating point number.
     */
    float    toFloat(const std::string &s) const;

    /**
     * @return @c "true" or @c "false".
     */
    std::string toString(bool i) const;

    /**
     * @return a decimal string representation of the given number @c i.
     */
    std::string toString(int i) const;

    /**
     * @return a decimal string representation of the given number @c i.
     */
    std::string toString(unsigned i) const;

    /**
     * @return a decimal string representation of the given number @c i.
     */
    std::string toString(int64_t i) const;

    /**
     * @return a decimal string representation of the given number @c i.
     */
    std::string toString(uint64_t i) const;

    /**
     * @return a decimal string representation of the given number @c i
     *         with precision 16.
     */
    std::string toString(double i) const;

    /**
     * @return a decimal string representation of the given number @c i
     *         with precision 8.
     */
    std::string toString(float i) const;

  private:
    std::locale loc;
  };
}

#endif // ! FANCYBASE_NUMERICCONVERTER_H
