/*
 * fancysharedlibrary.h
 *
 *  Created on: 24.01.2011
 *      Author: wglas
 */

#ifndef FANCYSHAREDLIBRARY_H_
#define FANCYSHAREDLIBRARY_H_

#include <fancyobject.h>

namespace Fancy {

/**
 * A shared library, which is resolved at runtime.
 */
class FANCYBASE_API SharedLibrary : virtual public Object {
protected:
    SharedLibrary();
    virtual ~SharedLibrary();

public:
    typedef ObjPtr<SharedLibrary> ptr_t;

    /**
     * Open a shared library by a an absolute or relative path.
     *
     * @param An operating system dependent path. If the path is relative, an
     *       operating system dependent library search algorithm is involved.
     * @throw Fancy::Exception, if the specified path could not be loaded as a shared library.
     * @see MSDN LoadLibraryA
     * @see dlopen(3)
     */
    static ptr_t newInstance(const char *path);

    /**
      * Open a shared library by a an absolute or relative path.
      *
      * @param An operating system dependent path. If the path is relative, an
      *       operating system dependent library search algorithm is involved.
      * @throw Fancy::Exception, if the specified path could not be loaded as a shared library.
      * @see MSDN LoadLibraryA
      * @see dlopen(3)
      */
     static ptr_t newInstance(const std::string &path);

    /**
     * Resolve a symbol from the shared library.
     *
     * @param sym The symbol name as seen by the runtime loader.
     * @return The symbol's address.
     * @throw Fancy::Exception, if the symbol could not be found inside the shared library.
     */
    virtual void *resolveSymbol(const char *sym) = 0;

    /**
     * Resolve a symbol from the shared library and cast it to the given pointer type.
     * There is no runtime type checking involved, because operating system interfaces to shared
     * libraries do not have a mechanism for checking the type of a resolved symbol.
     *
     * @param sym The symbol name as seen by the runtime loader.
     * @return The symbol's address casted to the pointer type <code>Ptr</code>.
     * @throw Fancy::Exception, if the symbol could not be found inside the shared library.
     */
    template<typename Ptr>
    Ptr resolveTypedSymbol(const char *sym) {
        return (Ptr)this->resolveSymbol(sym);
    }
};

}

#endif /* FANCYSHAREDLIBRARY_H_ */
