/*
 * fancyurl.h
 *
 *  Created on: 29.11.2009
 *      Author: wglas
 */

#ifndef FANCYURL_H_
#define FANCYURL_H_

#include <fancyserializable.h>

namespace Fancy {

class Map;
class String;
class Integer;

/**
 * An URL object used to parse URLs cf. to RFC 2396.
 */
class FANCYBASE_API URL: public virtual Fancy::Serializable {
public:
    typedef ObjPtr<URL> ptr_t;

    /**
     * Parse the given, encoded URL in the form
     * @verbatim
     * [<protocol>://][<user>@][<host>[:<port>]][/<path>[?<query>]][#<reference>]
     * @endverbatim
     * The individual parts may be quoted according to RFC 2396.
     *
     * @param s The encoded URL.
     * @return A parsed URL instance.
     */
    static ptr_t newInstance(const std::string &s);

    /**
     * Construct a new URL instance from the decoded parts of the URL.
     * @param protocol The protocol
     * @param user The user information.
     * @param host The host part.
     * @param port The port or @c -1, if no port is specified.
     * @param path The path part of the URL containing a leading slash.
     * @param parsedQuery The parsed query.
     * @param reference The reference part.
     * @return A new URL instance.
     */
    static ptr_t newInstance(const ObjPtr<String> &protocol, const ObjPtr<String> &user, const ObjPtr<String> &host,
                             int32_t port, const ObjPtr<String> &path, const ObjPtr<Map> &parsedQuery, const ObjPtr<String> &reference=ObjPtr<String>());

    /**
     * Construct a new URL instance from the decoded parts of the URL.
     * @param protocol The protocol
     * @param user The user information.
     * @param host The host part.
     * @param port The port or @c -1, if no port is specified.
     * @param path The path part of the URL containing a leading slash.
     * @param query The encoded query.
     * @param reference The reference part.
     * @return A new URL instance.
     */
    static ptr_t newInstance(const ObjPtr<String> &protocol, const ObjPtr<String> &user, const ObjPtr<String> &host,
                             int32_t port, const ObjPtr<String> &path, const ObjPtr<String> &query=ObjPtr<String>(), const ObjPtr<String> &reference=ObjPtr<String>());

    /**
     * Construct a new URL instance from the decoded parts of the URL.
     * @param protocol The protocol
     * @param host The host part.
     * @param port The port or @c -1, if no port is specified.
     * @param path The path part of the URL containing a leading slash.
     * @param parsedQuery The parsed query.
     * @param reference The reference part.
     * @return A new URL instance.
     */
    static ptr_t newInstance(const ObjPtr<String> &protocol, const ObjPtr<String> &host,
                             int32_t port, const ObjPtr<String> &path, const ObjPtr<Map> &parsedQuery, const ObjPtr<String> &reference=ObjPtr<String>());

    /**
     * Construct a new URL instance from the decoded parts of the URL.
     * @param protocol The protocol
     * @param host The host part.
     * @param port The port or @c -1, if no port is specified.
     * @param path The path part of the URL containing a leading slash.
     * @param query The encoded query.
     * @param reference The reference part.
     * @return A new URL instance.
     */
    static ptr_t newInstance(const ObjPtr<String> &protocol, const ObjPtr<String> &host,
                             int32_t port, const ObjPtr<String> &path, const ObjPtr<String> &query=ObjPtr<String>(), const ObjPtr<String> &reference=ObjPtr<String>());

    /**
     * Decode an URL part according to RFC 2396.
     * @param s An URL encoded string.
     * @return The decoded string.
     */
    static std::string decode(const std::string &s);

    /**
     * If this flag is set, spaces are encoded as plus signs.
     */
    static const int ENCODE_SPACES        = 1;
    /**
     * If this flag is set, slashes are encoded as @c %2F.
     */
    static const int ENCODE_SLASHES       = 2;
    /**
     * If this flag is set, slashes are encoded as @c %20.
     * If @c ENCODE_SPACES is set too, @c ENCODE_SPACES_AS_HEX
     * takes precedence over @c ENCODE_SPACES.
     */
    static const int ENCODE_SPACES_AS_HEX = 4;

    /**
     * Encode an URL part according to RFC 2396.
     * @param s An UTF-8 encoded string.
     * @param flags A bitwise combination of any of the flags
     *              @c ENCODE_SPACE, @c ENCODE_SLASHES or
     *              @c ENCODE_SPACES_AS_HEX.
     * @return The URL encoded string.
     */
    static std::string encode(const std::string &s, int flags=ENCODE_SLASHES|ENCODE_SPACES);

    /**
     * @return The decoded protocol of this URL or @c null,
     *         if no protocol has been specified.
     */
    virtual ObjPtr<String> getProtocol() const = 0;

    /**
     * @return The decoded user information part of this URL or @c null,
     *         if no authority has been specified.
     */
    virtual ObjPtr<String> getUser() const = 0;

   /**
     * @return The encoded authority part of this URL or @c null,
     *         if no authority has been specified.
     */
    virtual ObjPtr<String> getAuthority() const = 0;

    /**
     * @return The decoded host part of this URL or @c null,
     *         if no host has been specified.
     */
    virtual ObjPtr<String> getHost() const = 0;

    /**
     * @return The port of the URL or @c -1, if no port has been specified.
     */
    virtual int32_t        getPort() const = 0;

    /**
     * @return The decoded path of this URL or @c null,
     *         if no host has been specified. The path will always
     *         contain a leading slash character.
     */
    virtual ObjPtr<String> getPath() const = 0;

    /**
     * @return The encoded file part of this URL or @c null,
     *         if no file part has been specified. The file part
     *         merely exists in order to support JAVA interoperability.
     *         The file part is defined as @c path[?query].
     */
    virtual ObjPtr<String> getFile() const = 0;

    /**
     * @return The encoded query part of this URL or @c null,
     *         if no query part has been specified. If you do not wish
     *         to form-decode your query, you may decide to decode the
     *         result using @decode.
     */
    virtual ObjPtr<String> getQuery() const = 0;

    /**
     * @return The form-decoded query of this URL or @c null,
     *         if no query has been specified.
     *         The keys the given map are the parsed names of
     *         the query fields. The values are either a
     *         @c String instance containing the decoded value of the
     *         query field or a @c List instance, if a field name is
     *         specified multiple times. In the later case, the list
     *         contains all values of the specified field name.
     */
    virtual ObjPtr<Map>    getParsedQuery() const = 0;

    /**
     * @return The decoded reference of this URL or @c null,
     *         if no host has been specified.
     */
    virtual ObjPtr<String> getReference() const = 0;

    // implementation of Fancy::Serializable
    static Serializable::ptr_t newSerializableInstance();

    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;

protected:
    URL();
    virtual ~URL();
};

}

#endif /* FANCYURL_H_ */
